"""
SVM確率推定参考URL：https://scrapbox.io/nishio/SVM%E3%81%A7%E7%A2%BA%E7%8E%87%E6%8E%A8%E5%AE%9A
"""

import os

import numpy as np
from keras.applications.vgg16 import VGG16
from keras.applications.vgg16 import preprocess_input
from keras.preprocessing import image
from sklearn.externals import joblib

data_directry = "image"  # 今回学習する画像があるフォルダを指定
num_classes = 3  # 分類するクラス数

# 学習済み分類器の読み込み
clf = joblib.load('svc.pkl')
# print(clf) # パラメータを表示してみる

# 予測したい画像の読み込み
image_list = []  # 画像を収めるための空リスト
label_list = []  # 正解ラベルを収めるための空リスト
fullpath = []
for dir in os.listdir(data_directry):
    dir1 = data_directry + "/" + dir
    label = dir  # 画像が保存されているディレクトリの名前をそのまま正解ラベルとしている
    for f in os.listdir(dir1):
        label_list.append(label)
        filepath = dir1 + "/" + f
        fullpath.append((filepath))
        image_list.append(image.img_to_array(image.load_img(filepath)))
x = np.asarray(image_list)
x = preprocess_input(x)
y = np.asarray(label_list).astype(np.int)

# 特徴量の抽出
# 今回はVGG16の特徴量を用いて学習を行ったため、同様の形にする必要がある
base_model = VGG16(weights='imagenet', include_top=False, pooling="avg")
x_train_vgg16 = base_model.predict(x)

# 予測の実施
answer = []  # 正解ラベルとの比較結果を保存するリスト
pr_labels = clf.predict(x_train_vgg16)  # 単純にラベルを予測する
for i in range(len(pr_labels)):
    if (pr_labels[i] == y[i]):
        answer.append("o")
    else:
        answer.append("x")
pr_proba = clf.predict_proba(x_train_vgg16)  # どんな原理かよくわからないが、確率を求める

# 結果の保存
fout = open("predict.csv", "w")
fout.write("Name" + "," + "Correct" + "," + "Predict" + "," + "ox" + "," + "Probability [%]" + "\n")
for i in range(len(pr_labels)):
    fout.write(str(fullpath[i]) + "," + str(y[i]) + "," + str(pr_labels[i]) + "," + answer[i])
    for j in range(num_classes):
        fout.write("," + str(round(pr_proba[i][j] * 100, 1)))
    fout.write("\n")
fout.close()
