"""
参考URL：https://datahotel.io/archives/8352
VGG16の特徴量を利用した転移学習
"""

import os

import numpy as np
from keras.applications.vgg16 import VGG16
from keras.applications.vgg16 import preprocess_input
from keras.preprocessing import image
from sklearn.externals import joblib
from sklearn.metrics import accuracy_score
from sklearn.model_selection import train_test_split
from sklearn.svm import SVC
from sklearn.decomposition import PCA

data_directry = "image"  # 今回学習する画像があるフォルダを指定
num_classes = 3  # 分類するクラス数
batch_size = 10  # バッチのサイズ
epochs = 10  # 学習を行う回数

# 画像の読み込み
image_list = []  # 画像を収めるための空リスト
label_list = []  # 正解ラベルを収めるための空リスト
for dir in os.listdir(data_directry):
    dir1 = data_directry + "/" + dir
    label = dir  # 画像が保存されているディレクトリの名前をそのまま正解ラベルとしている
    for f in os.listdir(dir1):
        label_list.append(label)
        filepath = dir1 + "/" + f
        image_list.append(image.img_to_array(image.load_img(filepath)))

x = np.asarray(image_list)
x = preprocess_input(x)
y = np.asarray(label_list).astype(np.int)

x_train, x_test, y_train, y_test = train_test_split(x, y, test_size=0.33, random_state=412)
print(x_train.shape, x_test.shape, y_train.shape, y_test.shape)  # 変換後の入力配列の確認

### 転移学習 ###

# まずは基本となるVGG16のモデルとその特徴量を読み込む
# ただし、VGGの全結合層は不要なので、"include_top=False"で取り除き、最後に平均値でプーリングしている
base_model = VGG16(weights='imagenet', include_top=False, pooling="avg")

# 入力画像をVGG16に通し、新たな特徴量を抽出する
x_train_vgg16 = base_model.predict(x_train)
x_test_vgg16 = base_model.predict(x_test)

# 最後にSVMに学習させ、その結果を表示する
clf = SVC(kernel="linear", probability=True).fit(x_train_vgg16, y_train)
joblib.dump(clf, 'svc.pkl')
print("acc = {}".format(accuracy_score(clf.predict(x_train_vgg16), y_train)))
print("val_acc = {}".format(accuracy_score(clf.predict(x_test_vgg16), y_test)))
